@@ -61,6 +61,7 @@ class PlaybookFilter(DateFilter):
     status = django_filters.MultipleChoiceFilter(
         field_name="status", choices=ara_models.Playbook.STATUS, lookup_expr="iexact"
     )
+    label = django_filters.CharFilter(field_name="labels", lookup_expr="name__iexact")
 
     # fmt: off
     order = django_filters.OrderingFilter(
@@ -82,6 +83,7 @@ class PlayFilter(DateFilter):
     status = django_filters.MultipleChoiceFilter(
         field_name="status", choices=ara_models.Play.STATUS, lookup_expr="iexact"
     )
+    name = django_filters.CharFilter(field_name="name", lookup_expr="icontains")
 
     # fmt: off
     order = django_filters.OrderingFilter(
@@ -102,6 +104,10 @@ class TaskFilter(DateFilter):
     status = django_filters.MultipleChoiceFilter(
         field_name="status", choices=ara_models.Task.STATUS, lookup_expr="iexact"
     )
+    name = django_filters.CharFilter(field_name="name", lookup_expr="icontains")
+    action = django_filters.CharFilter(field_name="action", lookup_expr="iexact")
+    path = django_filters.CharFilter(field_name="file__path", lookup_expr="icontains")
+    handler = django_filters.BooleanFilter(field_name="handler", lookup_expr="exact")
 
     # fmt: off
     order = django_filters.OrderingFilter(
@@ -119,6 +125,19 @@ class TaskFilter(DateFilter):
 
 class HostFilter(BaseFilter):
     playbook = django_filters.NumberFilter(field_name="playbook__id", lookup_expr="exact")
+    name = django_filters.CharFilter(field_name="name", lookup_expr="icontains")
+
+    # For example: /api/v1/hosts/failed__gt=0 to return hosts with 1 failure or more
+    changed__gt = django_filters.NumberFilter(field_name="changed", lookup_expr="gt")
+    changed__lt = django_filters.NumberFilter(field_name="changed", lookup_expr="lt")
+    failed__gt = django_filters.NumberFilter(field_name="failed", lookup_expr="gt")
+    failed__lt = django_filters.NumberFilter(field_name="failed", lookup_expr="lt")
+    ok__gt = django_filters.NumberFilter(field_name="ok", lookup_expr="gt")
+    ok__lt = django_filters.NumberFilter(field_name="ok", lookup_expr="lt")
+    skipped__gt = django_filters.NumberFilter(field_name="skipped", lookup_expr="gt")
+    skipped__lt = django_filters.NumberFilter(field_name="skipped", lookup_expr="lt")
+    unreachable__gt = django_filters.NumberFilter(field_name="unreachable", lookup_expr="gt")
+    unreachable__lt = django_filters.NumberFilter(field_name="unreachable", lookup_expr="lt")
 
     # fmt: off
     order = django_filters.OrderingFilter(
@@ -139,9 +158,14 @@ class HostFilter(BaseFilter):
 
 class ResultFilter(DateFilter):
     playbook = django_filters.NumberFilter(field_name="playbook__id", lookup_expr="exact")
+    task = django_filters.NumberFilter(field_name="task__id", lookup_expr="exact")
+    play = django_filters.NumberFilter(field_name="play__id", lookup_expr="exact")
+    host = django_filters.NumberFilter(field_name="host__id", lookup_expr="exact")
+    changed = django_filters.BooleanFilter(field_name="changed", lookup_expr="exact")
     status = django_filters.MultipleChoiceFilter(
         field_name="status", choices=ara_models.Result.STATUS, lookup_expr="iexact"
     )
+    ignore_errors = django_filters.BooleanFilter(field_name="ignore_errors", lookup_expr="exact")
 
     # fmt: off
     order = django_filters.OrderingFilter(
@@ -159,7 +183,7 @@ class ResultFilter(DateFilter):
 
 class FileFilter(BaseFilter):
     playbook = django_filters.NumberFilter(field_name="playbook__id", lookup_expr="exact")
-    path = django_filters.CharFilter(field_name="path", lookup_expr="exact")
+    path = django_filters.CharFilter(field_name="path", lookup_expr="icontains")
 
     # fmt: off
     order = django_filters.OrderingFilter(
