@@ -66,7 +66,7 @@ class Label(Base):
     class Meta:
         db_table = "labels"
 
-    name = models.CharField(max_length=255)
+    name = models.CharField(max_length=255, unique=True)
 
     def __str__(self):
         return "<Label %s: %s>" % (self.id, self.name)
@@ -87,7 +87,14 @@ class Playbook(Duration):
     RUNNING = "running"
     COMPLETED = "completed"
     FAILED = "failed"
-    STATUS = ((UNKNOWN, "unknown"), (RUNNING, "running"), (COMPLETED, "completed"), (FAILED, "failed"))
+    EXPIRED = "expired"
+    STATUS = (
+        (UNKNOWN, "unknown"),
+        (EXPIRED, "expired"),
+        (RUNNING, "running"),
+        (COMPLETED, "completed"),
+        (FAILED, "failed"),
+    )
 
     name = models.CharField(max_length=255, null=True)
     ansible_version = models.CharField(max_length=255)
@@ -167,7 +174,8 @@ class Play(Duration):
     UNKNOWN = "unknown"
     RUNNING = "running"
     COMPLETED = "completed"
-    STATUS = ((UNKNOWN, "unknown"), (RUNNING, "running"), (COMPLETED, "completed"))
+    EXPIRED = "expired"
+    STATUS = ((UNKNOWN, "unknown"), (RUNNING, "running"), (COMPLETED, "completed"), (EXPIRED, "expired"))
 
     name = models.CharField(max_length=255, blank=True, null=True)
     uuid = models.UUIDField()
@@ -189,7 +197,8 @@ class Task(Duration):
     UNKNOWN = "unknown"
     RUNNING = "running"
     COMPLETED = "completed"
-    STATUS = ((UNKNOWN, "unknown"), (RUNNING, "running"), (COMPLETED, "completed"))
+    EXPIRED = "expired"
+    STATUS = ((UNKNOWN, "unknown"), (RUNNING, "running"), (COMPLETED, "completed"), (EXPIRED, "expired"))
 
     name = models.TextField(blank=True, null=True)
     action = models.TextField()
@@ -244,20 +253,18 @@ class Result(Duration):
     FAILED = "failed"
     SKIPPED = "skipped"
     UNREACHABLE = "unreachable"
-    # ARA specific statuses (derived or assumed)
-    CHANGED = "changed"
-    IGNORED = "ignored"
+    # ARA specific status, it's the default when not specified
     UNKNOWN = "unknown"
 
+    # fmt:off
     STATUS = (
         (OK, "ok"),
         (FAILED, "failed"),
         (SKIPPED, "skipped"),
         (UNREACHABLE, "unreachable"),
-        (CHANGED, "changed"),
-        (IGNORED, "ignored"),
         (UNKNOWN, "unknown"),
     )
+    # fmt:on
 
     status = models.CharField(max_length=25, choices=STATUS, default=UNKNOWN)
     changed = models.BooleanField(default=False)
