@@ -93,7 +93,7 @@ class PlaybookTestCase(APITestCase):
         self.assertEqual(201, request.status_code)
         self.assertEqual(1, models.Playbook.objects.count())
         self.assertEqual(request.data["status"], "running")
-        self.assertEqual(request.data["labels"], labels)
+        self.assertEqual(sorted([label["name"] for label in request.data["labels"]]), sorted(labels))
 
     def test_partial_update_playbook(self):
         playbook = factories.PlaybookFactory()
@@ -111,6 +111,15 @@ class PlaybookTestCase(APITestCase):
         playbook_updated = models.Playbook.objects.get(id=playbook.id)
         self.assertNotEqual("wrong", playbook_updated.status)
 
+    def test_expired_playbook(self):
+        playbook = factories.PlaybookFactory(status="running")
+        self.assertEqual("running", playbook.status)
+
+        request = self.client.patch("/api/v1/playbooks/%s" % playbook.id, {"status": "expired"})
+        self.assertEqual(200, request.status_code)
+        playbook_updated = models.Playbook.objects.get(id=playbook.id)
+        self.assertEqual("expired", playbook_updated.status)
+
     def test_get_playbook(self):
         playbook = factories.PlaybookFactory()
         request = self.client.get("/api/v1/playbooks/%s" % playbook.id)
@@ -226,4 +235,17 @@ class PlaybookTestCase(APITestCase):
         playbook_updated = models.Playbook.objects.get(id=playbook.id)
         self.assertEqual([label.name for label in playbook_updated.labels.all()], labels)
 
-    # TODO: Add tests for incrementally updating files
+    def test_get_playbook_by_label(self):
+        # Create two playbooks, one with labels and one without
+        playbook = factories.PlaybookFactory()
+        self.client.patch("/api/v1/playbooks/%s" % playbook.id, {"labels": ["test-label"]})
+        factories.PlaybookFactory()
+
+        # Ensure we have two objects when searching without labels
+        request = self.client.get("/api/v1/playbooks")
+        self.assertEqual(2, len(request.data["results"]))
+
+        # Search with label and ensure we have the right one
+        request = self.client.get("/api/v1/playbooks?label=%s" % "test-label")
+        self.assertEqual(1, len(request.data["results"]))
+        self.assertEqual(request.data["results"][0]["labels"][0]["name"], "test-label")
