@@ -59,7 +59,7 @@ class ResultTestCase(APITestCase):
         serializer = serializers.ResultSerializer(
             data={
                 "content": factories.RESULT_CONTENTS,
-                "status": "changed",
+                "status": "ok",
                 "host": host.id,
                 "task": task.id,
                 "play": task.play.id,
@@ -128,16 +128,33 @@ class ResultTestCase(APITestCase):
         request = self.client.get("/api/v1/results/%s" % result.id)
         self.assertEqual(result.status, request.data["status"])
 
-    def test_get_result_by_playbook(self):
-        playbook = factories.PlaybookFactory()
-        host_one = factories.HostFactory(name="one")
-        host_two = factories.HostFactory(name="two")
-        result = factories.ResultFactory(playbook=playbook, host=host_one, status="failed")
-        factories.ResultFactory(playbook=playbook, host=host_two, status="skipped")
-        request = self.client.get("/api/v1/results?playbook=%s" % playbook.id)
+    def test_get_result_by_association(self):
+        # Create two results in necessarily two different playbooks with different children:
+        # playbook -> play -> task -> result <- host
+        first_result = factories.ResultFactory()
+        second_result = factories.ResultFactory()
+
+        # the fields with the association ids
+        associations = ["playbook", "play", "task", "host"]
+
+        # Validate that we somehow didn't wind up with the same association ids
+        for association in associations:
+            first = getattr(first_result, association)
+            second = getattr(second_result, association)
+            self.assertNotEqual(first.id, second.id)
+
+        # In other words, there must be two distinct results
+        request = self.client.get("/api/v1/results")
+        self.assertEqual(2, request.data["count"])
         self.assertEqual(2, len(request.data["results"]))
-        self.assertEqual(result.status, request.data["results"][0]["status"])
-        self.assertEqual("skipped", request.data["results"][1]["status"])
+
+        # Searching for the first_result associations should only yield one result
+        for association in associations:
+            assoc_id = getattr(first_result, association).id
+            results = self.client.get("/api/v1/results?%s=%s" % (association, assoc_id))
+            self.assertEqual(1, results.data["count"])
+            self.assertEqual(1, len(results.data["results"]))
+            self.assertEqual(assoc_id, results.data["results"][0][association])
 
     def test_get_result_by_statuses(self):
         failed_result = factories.ResultFactory(status="failed")
@@ -156,8 +173,8 @@ class ResultTestCase(APITestCase):
 
         results = self.client.get("/api/v1/results?status=failed&status=skipped").data["results"]
         self.assertEqual(2, len(results))
-        self.assertEqual(failed_result.status, results[0]["status"])
-        self.assertEqual(skipped_result.status, results[1]["status"])
+        self.assertEqual(failed_result.status, results[1]["status"])
+        self.assertEqual(skipped_result.status, results[0]["status"])
 
     def test_result_status_serializer(self):
         ok = factories.ResultFactory(status="ok")
@@ -184,6 +201,24 @@ class ResultTestCase(APITestCase):
         result = self.client.get("/api/v1/results/%s" % unreachable.id)
         self.assertEqual(result.data["status"], "unreachable")
 
+    def test_get_result_with_ignore_errors(self):
+        failed = factories.ResultFactory(status="failed", ignore_errors=False)
+        ignored = factories.ResultFactory(status="failed", ignore_errors=True)
+
+        # Searching for failed should return both
+        results = self.client.get("/api/v1/results?status=failed").data["results"]
+        self.assertEqual(2, len(results))
+
+        # Searching for failed with ignore_errors=True should only return the ignored result
+        results = self.client.get("/api/v1/results?status=failed&ignore_errors=true").data["results"]
+        self.assertEqual(1, len(results))
+        self.assertEqual(ignored.id, results[0]["id"])
+
+        # Searching for failed with ignore_errors=False should only return the failed result
+        results = self.client.get("/api/v1/results?status=failed&ignore_errors=false").data["results"]
+        self.assertEqual(1, len(results))
+        self.assertEqual(failed.id, results[0]["id"])
+
     def test_get_result_duration(self):
         started = timezone.now()
         ended = started + datetime.timedelta(hours=1)
@@ -231,3 +266,21 @@ class ResultTestCase(APITestCase):
         for field in order_fields:
             request = self.client.get("/api/v1/results?order=-%s" % field)
             self.assertEqual(request.data["results"][0]["id"], new_result.id)
+
+    def test_get_changed_results(self):
+        changed_result = factories.ResultFactory(changed=True)
+        unchanged_result = factories.ResultFactory(changed=False)
+
+        # Assert two results
+        results = self.client.get("/api/v1/results").data["results"]
+        self.assertEqual(2, len(results))
+
+        # Assert one changed
+        results = self.client.get("/api/v1/results?changed=true").data["results"]
+        self.assertEqual(1, len(results))
+        self.assertEqual(results[0]["id"], changed_result.id)
+
+        # Assert one unchanged
+        results = self.client.get("/api/v1/results?changed=false").data["results"]
+        self.assertEqual(1, len(results))
+        self.assertEqual(results[0]["id"], unchanged_result.id)
