@@ -122,6 +122,15 @@ class TaskTestCase(APITestCase):
         task_updated = models.Task.objects.get(id=task.id)
         self.assertEqual("update", task_updated.name)
 
+    def test_expired_task(self):
+        task = factories.TaskFactory(status="running")
+        self.assertEqual("running", task.status)
+
+        request = self.client.patch("/api/v1/tasks/%s" % task.id, {"status": "expired"})
+        self.assertEqual(200, request.status_code)
+        task_updated = models.Task.objects.get(id=task.id)
+        self.assertEqual("expired", task_updated.status)
+
     def test_get_task(self):
         task = factories.TaskFactory()
         request = self.client.get("/api/v1/tasks/%s" % task.id)
@@ -133,8 +142,19 @@ class TaskTestCase(APITestCase):
         factories.TaskFactory(name="task2", playbook=playbook)
         request = self.client.get("/api/v1/tasks?playbook=%s" % playbook.id)
         self.assertEqual(2, len(request.data["results"]))
+        self.assertEqual(task.name, request.data["results"][1]["name"])
+        self.assertEqual("task2", request.data["results"][0]["name"])
+
+    def test_get_tasks_by_name(self):
+        # Create a playbook and two tasks
+        playbook = factories.PlaybookFactory()
+        task = factories.TaskFactory(name="task1", playbook=playbook)
+        factories.TaskFactory(name="task2", playbook=playbook)
+
+        # Query for the first task name and expect one result
+        request = self.client.get("/api/v1/tasks?name=%s" % task.name)
+        self.assertEqual(1, len(request.data["results"]))
         self.assertEqual(task.name, request.data["results"][0]["name"])
-        self.assertEqual("task2", request.data["results"][1]["name"])
 
     def test_get_task_duration(self):
         started = timezone.now()
@@ -209,3 +229,35 @@ class TaskTestCase(APITestCase):
         # Test multiple status
         request = self.client.get("/api/v1/tasks?status=running&status=completed")
         self.assertEqual(2, len(request.data["results"]))
+
+    def test_get_task_by_action(self):
+        task = factories.TaskFactory(action="debug")
+        factories.TaskFactory(action="setup")
+
+        # Confirm we have two objects
+        request = self.client.get("/api/v1/tasks")
+        self.assertEqual(2, len(request.data["results"]))
+
+        # Expect the correct single result when searching
+        request = self.client.get("/api/v1/tasks?action=debug")
+        self.assertEqual(1, len(request.data["results"]))
+        self.assertEqual(task.id, request.data["results"][0]["id"])
+        self.assertEqual(task.action, request.data["results"][0]["action"])
+
+    def test_get_task_by_path(self):
+        # Create two files with different paths
+        first_file = factories.FileFactory(path="/root/roles/foo/tasks/main.yml")
+        second_file = factories.FileFactory(path="/root/roles/bar/tasks/main.yml")
+
+        # Create two tasks using these files
+        task = factories.TaskFactory(file=first_file)
+        factories.TaskFactory(file=second_file)
+
+        # Test exact match
+        request = self.client.get("/api/v1/tasks?path=/root/roles/foo/tasks/main.yml")
+        self.assertEqual(1, len(request.data["results"]))
+        self.assertEqual(task.file.path, request.data["results"][0]["path"])
+
+        # Test partial match
+        request = self.client.get("/api/v1/tasks?path=main.yml")
+        self.assertEqual(len(request.data["results"]), 2)
