@@ -21,10 +21,12 @@ import datetime
 import json
 import logging
 import os
+from concurrent.futures import ThreadPoolExecutor
 
-import six
 from ansible import __version__ as ansible_version
+from ansible.parsing.ajson import AnsibleJSONEncoder
 from ansible.plugins.callback import CallbackBase
+from ansible.vars.clean import module_response_deepcopy, strip_internal_keys
 
 from ara.clients import utils as client_utils
 
@@ -105,6 +107,32 @@ options:
     ini:
       - section: ara
         key: api_timeout
+  argument_labels:
+    description: |
+        A list of CLI arguments that, if set, will be automatically applied to playbooks as labels.
+        Note that CLI arguments are not always named the same as how they are represented by Ansible.
+        For example, --limit is "subset", --user is "remote_user" but --check is "check".
+    type: list
+    default:
+      - remote_user
+      - check
+      - tags
+      - skip_tags
+      - subset
+    env:
+      - name: ARA_ARGUMENT_LABELS
+    ini:
+      - section: ara
+        key: argument_labels
+  default_labels:
+    description: A list of default labels that will be applied to playbooks
+    type: list
+    default: []
+    env:
+      - name: ARA_DEFAULT_LABELS
+    ini:
+      - section: ara
+        key: default_labels
   ignored_facts:
     description: List of host facts that will not be saved by ARA
     type: list
@@ -123,6 +151,15 @@ options:
     ini:
       - section: ara
         key: ignored_arguments
+  ignored_files:
+    description: List of patterns that will not be saved by ARA
+    type: list
+    default: []
+    env:
+      - name: ARA_IGNORED_FILES
+    ini:
+      - section: ara
+        key: ignored_files
 """
 
 
@@ -139,23 +176,35 @@ class CallbackModule(CallbackBase):
         super(CallbackModule, self).__init__()
         self.log = logging.getLogger("ara.plugins.callback.default")
         self.client = None
+        # TODO: Consider un-hardcoding this and plumbing pool_maxsize to requests.adapters.HTTPAdapter.
+        #       In the meantime default to 10 because that's the value of requests.adapters.DEFAULT_POOLSIZE.
+        #       Otherwise we can hit "urllib3.connectionpool: Connection pool is full"
+        self.thread_count = 10
+        self.global_threads = ThreadPoolExecutor(max_workers=self.thread_count)
+        # Need individual threads for tasks to ensure all results are saved before moving on to next task
+        self.task_threads = None
         self.ignored_facts = []
         self.ignored_arguments = []
+        self.ignored_files = []
 
         self.result = None
+        self.result_started = {}
+        self.result_ended = {}
         self.task = None
         self.play = None
         self.playbook = None
         self.stats = None
-        self.loop_items = []
         self.file_cache = {}
         self.host_cache = {}
 
     def set_options(self, task_keys=None, var_options=None, direct=None):
         super(CallbackModule, self).set_options(task_keys=task_keys, var_options=var_options, direct=direct)
 
+        self.argument_labels = self.get_option("argument_labels")
+        self.default_labels = self.get_option("default_labels")
         self.ignored_facts = self.get_option("ignored_facts")
         self.ignored_arguments = self.get_option("ignored_arguments")
+        self.ignored_files = self.get_option("ignored_files")
 
         client = self.get_option("api_client")
         endpoint = self.get_option("api_server")
@@ -174,7 +223,16 @@ class CallbackModule(CallbackBase):
 
     def v2_playbook_on_start(self, playbook):
         self.log.debug("v2_playbook_on_start")
-        path = os.path.abspath(playbook._file_name)
+
+        content = None
+
+        if playbook._file_name == "__adhoc_playbook__":
+            content = cli_options["module_name"]
+            if cli_options["module_args"]:
+                content = "{0}: {1}".format(content, cli_options["module_args"])
+            path = "Ad-Hoc: {0}".format(content)
+        else:
+            path = os.path.abspath(playbook._file_name)
 
         # Potentially sanitize some user-specified keys
         for argument in self.ignored_arguments:
@@ -182,6 +240,23 @@ class CallbackModule(CallbackBase):
                 self.log.debug("Ignoring argument: %s" % argument)
                 cli_options[argument] = "Not saved by ARA as configured by 'ignored_arguments'"
 
+        # Retrieve and format CLI options for argument labels
+        argument_labels = []
+        for label in self.argument_labels:
+            if label in cli_options:
+                # Some arguments are lists or tuples
+                if isinstance(cli_options[label], tuple) or isinstance(cli_options[label], list):
+                    # Only label these if they're not empty
+                    if cli_options[label]:
+                        argument_labels.append("%s:%s" % (label, ",".join(cli_options[label])))
+                # Some arguments are booleans
+                elif isinstance(cli_options[label], bool):
+                    argument_labels.append("%s:%s" % (label, cli_options[label]))
+                # The rest can be printed as-is if there is something set
+                elif cli_options[label]:
+                    argument_labels.append("%s:%s" % (label, cli_options[label]))
+        self.argument_labels = argument_labels
+
         # Create the playbook
         self.playbook = self.client.post(
             "/api/v1/playbooks",
@@ -193,7 +268,7 @@ class CallbackModule(CallbackBase):
         )
 
         # Record the playbook file
-        self._get_or_create_file(path)
+        self.global_threads.submit(self._get_or_create_file, path, content)
 
         return self.playbook
 
@@ -206,12 +281,24 @@ class CallbackModule(CallbackBase):
         play_vars = play._variable_manager.get_vars(play=play)["vars"]
         if "ara_playbook_name" in play_vars:
             self._set_playbook_name(name=play_vars["ara_playbook_name"])
+
+        labels = self.default_labels + self.argument_labels
         if "ara_playbook_labels" in play_vars:
-            self._set_playbook_labels(labels=play_vars["ara_playbook_labels"])
+            # ara_playbook_labels can be supplied as a list inside a playbook
+            # but it might also be specified as a comma separated string when
+            # using extra-vars
+            if isinstance(play_vars["ara_playbook_labels"], list):
+                labels.extend(play_vars["ara_playbook_labels"])
+            elif isinstance(play_vars["ara_playbook_labels"], str):
+                labels.extend(play_vars["ara_playbook_labels"].split(","))
+            else:
+                raise TypeError("ara_playbook_labels must be a list or a comma-separated string")
+        if labels:
+            self._set_playbook_labels(labels=labels)
 
         # Record all the files involved in the play
         for path in play._loader._FILE_CACHE.keys():
-            self._get_or_create_file(path)
+            self.global_threads.submit(self._get_or_create_file, path)
 
         # Create the play
         self.play = self.client.post(
@@ -225,9 +312,15 @@ class CallbackModule(CallbackBase):
 
         return self.play
 
+    def v2_playbook_on_handler_task_start(self, task):
+        self.log.debug("v2_playbook_on_handler_task_start")
+        # TODO: Why doesn't `v2_playbook_on_handler_task_start` have is_conditional ?
+        self.v2_playbook_on_task_start(task, False, handler=True)
+
     def v2_playbook_on_task_start(self, task, is_conditional, handler=False):
         self.log.debug("v2_playbook_on_task_start")
         self._end_task()
+        self.task_threads = ThreadPoolExecutor(max_workers=self.thread_count)
 
         pathspec = task.get_path()
         if pathspec:
@@ -257,17 +350,21 @@ class CallbackModule(CallbackBase):
 
         return self.task
 
+    def v2_runner_on_start(self, host, task):
+        # v2_runner_on_start was added in 2.8 so this doesn't get run for Ansible 2.7 and below.
+        self.result_started[host.get_name()] = datetime.datetime.now().isoformat()
+
     def v2_runner_on_ok(self, result, **kwargs):
-        self._load_result(result, "ok", **kwargs)
+        self.task_threads.submit(self._load_result, result, "ok", **kwargs)
 
     def v2_runner_on_unreachable(self, result, **kwargs):
-        self._load_result(result, "unreachable", **kwargs)
+        self.task_threads.submit(self._load_result, result, "unreachable", **kwargs)
 
     def v2_runner_on_failed(self, result, **kwargs):
-        self._load_result(result, "failed", **kwargs)
+        self.task_threads.submit(self._load_result, result, "failed", **kwargs)
 
     def v2_runner_on_skipped(self, result, **kwargs):
-        self._load_result(result, "skipped", **kwargs)
+        self.task_threads.submit(self._load_result, result, "skipped", **kwargs)
 
     def v2_playbook_on_stats(self, stats):
         self.log.debug("v2_playbook_on_stats")
@@ -278,16 +375,24 @@ class CallbackModule(CallbackBase):
 
     def _end_task(self):
         if self.task is not None:
-            self.client.patch(
-                "/api/v1/tasks/%s" % self.task["id"], status="completed", ended=datetime.datetime.now().isoformat()
+            self.task_threads.submit(
+                self.client.patch,
+                "/api/v1/tasks/%s" % self.task["id"],
+                status="completed",
+                ended=datetime.datetime.now().isoformat(),
             )
+            # Flush threads before moving on to next task to make sure all results are saved
+            self.task_threads.shutdown(wait=True)
+            self.task_threads = None
             self.task = None
-            self.loop_items = []
 
     def _end_play(self):
         if self.play is not None:
-            self.client.patch(
-                "/api/v1/plays/%s" % self.play["id"], status="completed", ended=datetime.datetime.now().isoformat()
+            self.global_threads.submit(
+                self.client.patch,
+                "/api/v1/plays/%s" % self.play["id"],
+                status="completed",
+                ended=datetime.datetime.now().isoformat(),
             )
             self.play = None
 
@@ -298,28 +403,39 @@ class CallbackModule(CallbackBase):
         else:
             status = "completed"
 
-        self.playbook = self.client.patch(
-            "/api/v1/playbooks/%s" % self.playbook["id"], status=status, ended=datetime.datetime.now().isoformat()
+        self.global_threads.submit(
+            self.client.patch,
+            "/api/v1/playbooks/%s" % self.playbook["id"],
+            status=status,
+            ended=datetime.datetime.now().isoformat(),
         )
+        self.global_threads.shutdown(wait=True)
 
     def _set_playbook_name(self, name):
         if self.playbook["name"] != name:
             self.playbook = self.client.patch("/api/v1/playbooks/%s" % self.playbook["id"], name=name)
 
     def _set_playbook_labels(self, labels):
-        if self.playbook["labels"] != labels:
+        current_labels = [label["name"] for label in self.playbook["labels"]]
+        if sorted(current_labels) != sorted(labels):
+            self.log.debug("Updating playbook labels to match: %s" % ",".join(labels))
             self.playbook = self.client.patch("/api/v1/playbooks/%s" % self.playbook["id"], labels=labels)
 
-    def _get_or_create_file(self, path):
+    def _get_or_create_file(self, path, content=None):
         if path not in self.file_cache:
             self.log.debug("File not in cache, getting or creating: %s" % path)
-            try:
-                with open(path, "r") as fd:
-                    content = fd.read()
-            except IOError as e:
-                self.log.error("Unable to open {0} for reading: {1}".format(path, six.text_type(e)))
-                content = """ARA was not able to read this file successfully.
-                        Refer to the logs for more information"""
+            for ignored_file_pattern in self.ignored_files:
+                if ignored_file_pattern in path:
+                    self.log.debug("Ignoring file {1}, matched pattern: {0}".format(ignored_file_pattern, path))
+                    content = "Not saved by ARA as configured by 'ignored_files'"
+            if content is None:
+                try:
+                    with open(path, "r") as fd:
+                        content = fd.read()
+                except IOError as e:
+                    self.log.error("Unable to open {0} for reading: {1}".format(path, str(e)))
+                    content = """ARA was not able to read this file successfully.
+                            Refer to the logs for more information"""
 
             self.file_cache[path] = self.client.post(
                 "/api/v1/files", playbook=self.playbook["id"], path=path, content=content
@@ -340,23 +456,30 @@ class CallbackModule(CallbackBase):
         host completes. It is responsible for logging a single result to the
         database.
         """
-        # Retrieve the host so we can associate the result to the host id
-        host = self._get_or_create_host(result._host.get_name())
+        hostname = result._host.get_name()
+        self.result_ended[hostname] = datetime.datetime.now().isoformat()
 
-        # Use Ansible's CallbackBase._dump_results in order to strip internal
-        # keys, respect no_log directive, etc.
-        if self.loop_items:
-            # NOTE (dmsimard): There is a known issue in which Ansible can send
-            # callback hooks out of order and "exit" the task before all items
-            # have returned, this can cause one of the items to be missing
-            # from the task result in ARA.
-            # https://github.com/ansible/ansible/issues/24207
-            results = [self._dump_results(result._result)]
-            for item in self.loop_items:
-                results.append(self._dump_results(item._result))
-            results = json.loads(json.dumps(results))
-        else:
-            results = json.loads(self._dump_results(result._result))
+        # Retrieve the host so we can associate the result to the host id
+        host = self._get_or_create_host(hostname)
+
+        results = strip_internal_keys(module_response_deepcopy(result._result))
+
+        # Round-trip through JSON to sort keys and convert Ansible types
+        # to standard types
+        try:
+            jsonified = json.dumps(results, cls=AnsibleJSONEncoder, ensure_ascii=False, sort_keys=True)
+        except TypeError:
+            # Python 3 can't sort non-homogenous keys.
+            # https://bugs.python.org/issue25457
+            jsonified = json.dumps(results, cls=AnsibleJSONEncoder, ensure_ascii=False, sort_keys=False)
+        results = json.loads(jsonified)
+
+        # Sanitize facts
+        if "ansible_facts" in results:
+            for fact in self.ignored_facts:
+                if fact in results["ansible_facts"]:
+                    self.log.debug("Ignoring fact: %s" % fact)
+                    results["ansible_facts"][fact] = "Not saved by ARA as configured by 'ignored_facts'"
 
         self.result = self.client.post(
             "/api/v1/results",
@@ -366,21 +489,14 @@ class CallbackModule(CallbackBase):
             play=self.task["play"],
             content=results,
             status=status,
-            started=self.task["started"],
-            ended=datetime.datetime.now().isoformat(),
+            started=self.result_started[hostname] if hostname in self.result_started else self.task["started"],
+            ended=self.result_ended[hostname],
             changed=result._result.get("changed", False),
             # Note: ignore_errors might be None instead of a boolean
             ignore_errors=kwargs.get("ignore_errors", False) or False,
         )
 
         if self.task["action"] in ["setup", "gather_facts"] and "ansible_facts" in results:
-            # Potentially sanitize some Ansible facts to prevent them from
-            # being saved both in the host facts and in the task results.
-            for fact in self.ignored_facts:
-                if fact in results["ansible_facts"]:
-                    self.log.debug("Ignoring fact: %s" % fact)
-                    results["ansible_facts"][fact] = "Not saved by ARA as configured by 'ignored_facts'"
-
             self.client.patch("/api/v1/hosts/%s" % host["id"], facts=results["ansible_facts"])
 
     def _load_stats(self, stats):
@@ -388,7 +504,9 @@ class CallbackModule(CallbackBase):
         for hostname in hosts:
             host = self._get_or_create_host(hostname)
             host_stats = stats.summarize(hostname)
-            self.client.patch(
+
+            self.global_threads.submit(
+                self.client.patch,
                 "/api/v1/hosts/%s" % host["id"],
                 changed=host_stats["changed"],
                 unreachable=host_stats["unreachable"],
