@@ -20,15 +20,24 @@ import textwrap
 import warnings
 
 import tzlocal
-import yaml
 from django.utils.crypto import get_random_string
 from dynaconf import LazySettings
 
+# dynaconf prefers ruamel.yaml but works with pyyaml
+# https://github.com/rochacbruno/dynaconf/commit/d5cf87cbbdf54625ccf1138a4e4c210956791e61
+try:
+    from ruamel import yaml as yaml
+except ImportError:
+    import yaml
+
 BASE_DIR = os.environ.get("ARA_BASE_DIR", os.path.expanduser("~/.ara/server"))
 DEFAULT_SETTINGS = os.path.join(BASE_DIR, "settings.yaml")
 
 settings = LazySettings(
-    GLOBAL_ENV_FOR_DYNACONF="ARA", ENVVAR_FOR_DYNACONF="ARA_SETTINGS", SETTINGS_MODULE_FOR_DYNACONF=DEFAULT_SETTINGS
+    environments=True,
+    GLOBAL_ENV_FOR_DYNACONF="ARA",
+    ENVVAR_FOR_DYNACONF="ARA_SETTINGS",
+    SETTINGS_MODULE_FOR_DYNACONF=DEFAULT_SETTINGS,
 )
 
 # reread BASE_DIR since it might have gotten changed in the config file.
@@ -117,6 +126,7 @@ DATABASE_PASSWORD = settings.get("DATABASE_PASSWORD", None)
 DATABASE_HOST = settings.get("DATABASE_HOST", None)
 DATABASE_PORT = settings.get("DATABASE_PORT", None)
 DATABASE_CONN_MAX_AGE = settings.get("DATABASE_CONN_MAX_AGE", 0)
+DATABASE_OPTIONS = settings.get("DATABASE_OPTIONS", {})
 
 DATABASES = {
     "default": {
@@ -127,6 +137,7 @@ DATABASES = {
         "HOST": DATABASE_HOST,
         "PORT": DATABASE_PORT,
         "CONN_MAX_AGE": DATABASE_CONN_MAX_AGE,
+        "OPTIONS": DATABASE_OPTIONS,
     }
 }
 
@@ -150,6 +161,9 @@ EXTERNAL_AUTH_MIDDLEWARE = []
 if EXTERNAL_AUTH:
     EXTERNAL_AUTH_MIDDLEWARE = ["django.contrib.auth.middleware.RemoteUserMiddleware"]
     AUTHENTICATION_BACKENDS = ["django.contrib.auth.backends.RemoteUserBackend"]
+    REST_FRAMEWORK_AUTH = ("rest_framework.authentication.RemoteUserAuthentication",)
+else:
+    REST_FRAMEWORK_AUTH = ("rest_framework.authentication.BasicAuthentication",)
 
 # fmt: off
 MIDDLEWARE = [
@@ -214,6 +228,7 @@ ROOT_URLCONF = "ara.server.urls"
 APPEND_SLASH = False
 
 PAGE_SIZE = settings.get("PAGE_SIZE", 100)
+
 REST_FRAMEWORK = {
     "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.LimitOffsetPagination",
     "PAGE_SIZE": PAGE_SIZE,
@@ -227,7 +242,7 @@ REST_FRAMEWORK = {
         "rest_framework.parsers.FormParser",
         "rest_framework.parsers.MultiPartParser",
     ),
-    "DEFAULT_AUTHENTICATION_CLASSES": ("rest_framework.authentication.BasicAuthentication",),
+    "DEFAULT_AUTHENTICATION_CLASSES": REST_FRAMEWORK_AUTH,
     "DEFAULT_PERMISSION_CLASSES": ("ara.api.auth.APIAccessPermission",),
     "TEST_REQUEST_DEFAULT_FORMAT": "json",
     "UNICODE_JSON": False,
@@ -245,10 +260,11 @@ if not os.path.isdir(BASE_DIR):
 if not os.path.exists(DEFAULT_SETTINGS) and "ARA_SETTINGS" not in os.environ:
     SETTINGS = dict(
         BASE_DIR=BASE_DIR,
-        ALLOWED_HOSTS=ALLOWED_HOSTS,
-        CORS_ORIGIN_WHITELIST=CORS_ORIGIN_WHITELIST,
-        CORS_ORIGIN_REGEX_WHITELIST=CORS_ORIGIN_REGEX_WHITELIST,
+        ALLOWED_HOSTS=ALLOWED_HOSTS.to_list(),
+        CORS_ORIGIN_WHITELIST=CORS_ORIGIN_WHITELIST.to_list(),
+        CORS_ORIGIN_REGEX_WHITELIST=CORS_ORIGIN_REGEX_WHITELIST.to_list(),
         CORS_ORIGIN_ALLOW_ALL=CORS_ORIGIN_ALLOW_ALL,
+        EXTERNAL_AUTH=EXTERNAL_AUTH,
         SECRET_KEY=SECRET_KEY,
         DATABASE_ENGINE=DATABASE_ENGINE,
         DATABASE_NAME=DATABASE_NAME,
@@ -257,6 +273,7 @@ if not os.path.exists(DEFAULT_SETTINGS) and "ARA_SETTINGS" not in os.environ:
         DATABASE_HOST=DATABASE_HOST,
         DATABASE_PORT=DATABASE_PORT,
         DATABASE_CONN_MAX_AGE=DATABASE_CONN_MAX_AGE,
+        DATABASE_OPTIONS=DATABASE_OPTIONS.to_dict(),
         DEBUG=DEBUG,
         LOG_LEVEL=LOG_LEVEL,
         LOGGING=LOGGING,
